pub mod device_messages;
pub mod nibp_messages;
pub mod temperature_messages;
pub mod tymp_messages;
use crate::{datatypes,object_species};

pub trait Message{
    fn get_raw_id(&self) -> u32;
    fn get_family(&self) -> datatypes::Family;
    fn get_genus(&self) ->  datatypes::Genus;
    fn get_species(&self) -> Box<dyn MessageSpecies>;
    fn get_length(&self) -> u32;
    fn get_bitmask(&self) -> u8;
    fn get_object_size(&self) -> u32;
    fn get_object(&self) -> Option<Box<dyn object_species::Object>>;
    fn get_crc(&self) -> u16;
}

pub trait MessageSpecies{}

//Messages relating to a device's battery
impl MessageSpecies for BatteryMessage{}
pub enum BatteryMessage{
    GetBattery,             //Request   0x00; Requires BatterySelectobj, defaults to Battery
    PutBattery,             //Response  0x00; Requires requested battery obj
    WriteBatteryStatus,     //Command   0x00; Requires WACP Numeric representing BatteryStatus
                            //                  enum(?)
    ReportBattery           //Status    0x00; Requires battery obj(?)
}

//Messages relating to BMI information
impl MessageSpecies for BMIMessage{}
pub enum BMIMessage{
    GetBMI,                 //Request   0x00; no obj(?)
    GetConfig,              //Request   0x01; no obj(?)
    PutBMI,                 //Response  0x00; requires BMIData obj (?)
    PutConfig,              //Response  0x01; requires BMIConfig obj (?)
    WriteConfig,            //Command   0x00; requires BMIConfig obj (?)
    ReportBMI,              //Status    0x00; no obj(?)
    StreamBMI               //Stream    0x00; requires BMIData obj(?)
}

//Messages relating to clinician information
impl MessageSpecies for ClinicianMessage{}
pub enum ClinicianMessage{
    GetClinician,           //Request   0x00; no obj(?)
    GetConfig,              //Request   0x01; no obj(?)
    PutClinician,           //Response  0x00; requires ClinicianData obj (?)
    PutConfig,              //Response  0x01; requires ClinicianConfig obj (?)
    WriteConfig,            //Command   0x00; requires ClinicianConfig obj (?)
    WriteData,              //Status    0x00; requires ClinicianConfig obj (?)
    ReportClinician         //Stream    0x00; requires ClinicianData obj(?)
}

//Messages relating to the CO2 sensor
impl MessageSpecies for CO2Message{}
pub enum CO2Message{
    GetBreath,              //Request   0x00; No obj
    PutBreath,              //Response  0x00; Requires CO2Data obj
    StreamCO2,              //Stream    0x00; Requires CO2Data obj
    RestartBreathAlg,       //Command   0x00; No obj
    GetAmbientPressure,     //Request   0x01; No obj
    PutAmbientPressure,     //Response  0x01; Requires CO2MsgArg obj
    GetModuleConfig,        //Request   0x02; Requires CO2MsgArg with IOParamA set to the
                            //                  ConfigItem to return
    PutModuleConfig,        //Response  0x02; Requires C02MsgArg with IOParamA set to the
                            //                  ConfigItem, and IOParamB set to the value
    StartCalib,             //Command   0x01; Requires CO2Cal obj
    StopCalib,              //Command   0x02; no obj
    GetServiceTime,         //Request   0x03; no obj
    PutServiceTime,         //Response  0x03; Requires CO2Cal obj
    ForceSFM,               //Command   0x03; No obj
    PumpControl,            //Command   0x04; Requires CO2MsgArg obj
    StartSFM,               //Command   0x05; No obj
    SetModuleConfig,        //Command   0x06; Requires C02MsgArg with IOParamA set to the
                            //                  ConfigItem to modify, and IOParamB set to the value
    GetCalTime,             //Request   0x04; No obj
    PutCalTime,             //Response  0x04; Requires CO2MsgArg with IOParamA set to remaining
                            //                  hours to calibration
    StartAcquire,           //Command   0x07; no obj(?)
    StopAcquire,            //Command   0x08; no obj(?)
    GetConfig,              //Request   0x05; no obj(?)
    PutConfig,              //Response  0x05; Requires CO2Config obj
    WriteConfig,            //Command   0x09; Requires CO2Config obj
}

//Messages containing collections
pub enum CollectionMessage{
    GetCollection,          //Request   0x00; no obj(?)
    PutCollection,          //Response  0x00; Some collection obj(?)
    WriteCollection,        //Command   0x00; Requires CollectConfig obj(?)
    ReportCollection,       //Status    0x00; Requires CollectConfigObj(?)
}

//Messages relating to commandline usage
impl MessageSpecies for CmdlineMessage{}
pub enum CmdlineMessage{
    GetConfig,              //Request   0x00; no obj(?)
    PutConfig,              //Response  0x00; some obj???
    GetData,                //Request   0x01; no obj(?)
    PutData,                //Response  0x01; some CmdlineData obj?
    Execute,                //Command   0x00; ???
    Report,                 //Status    0x00; ???
}

//Messages relating to data transfer
impl MessageSpecies for DataTransferMessage{}
pub enum DataTransferMessage{
    GetConfig,              //Request   0x00; Requires DataTransferDesc obj
    PutConfig,              //Response  0x00; Requires DataTransferDesc obj
    GetData,                //Request   0x01; Requires DataTransfer obj
    PutData,                //Response  0x01; Requires DataTransfer obj
    ReportDTR,              //Status    0x00; Requires DataTransfer obj
    ReportDTC,              //Status    0x01; Requires DataTransfer obj
    ExecuteDTT,             //Command   0x00; Requires DataTransfer obj
}

//Messages relating to the ECG module
impl MessageSpecies for ECGMessage{}
pub enum ECGMessage{
    GetConfig,              //Request   0x00; no obj
    PutConfig,              //Response  0x00; some ECG config obj???
    WriteConfig,            //Command   0x00; some ECG Config obj???
    GetLeadMeasurement,     //Request   0x01;
    PutLeadMeasurement,     //Response  0x01;
    GetMeasurements,        //Request   0x02;
    PutMeasurements,        //Response  0x02;
    GetLeadImpedance,       //Request   0x03;
    PutLeadImpedance,       //Response  0x03;
    GetQRSComplex,          //Request   0x04;
    PutQRSComplex,          //Response  0x04;
    StartTest,              //Command   0x01;
    StopTest,               //Command   0x02;
    ECGMode,                //Command   0x03;
}

//Error messages
impl MessageSpecies for ErrorMessage{}
pub enum ErrorMessage{
    GetError,               //Request   0x00; Requires GetErr obj
    PutError,               //Response  0x00; Requires Err obj
    ReportError,            //Status    0x00; requires Err obj; requires ExtErr obj containing 1+
                            //                  Err obj for devices with container support
    ClearError,             //Command   0x00; requires Err obj
}

//Messages relating to the device's filesystem
impl MessageSpecies for FilesysMessage{}
pub enum FilesysMessage{
    GetFile,                //Request   0x00; requires FileDesc obj [LEGACY]
    GetFileCount,           //Request   0x01; requires FilesysSelect obj
    PutFile,                //Response  0x00; Requires File obj [LEGACY]
    PutFileCount,           //Response  0x01; requires Numeric obj containing count
    WriteFile,              //Command   0x00; requires File obj [LEGACY]
    GetFilelist,            //Request   0x02; requires FilesysSelect obj
    PutFileList,            //Response  0x02; requires ExtFileData containing 1+ FileDesc obj
    CreateFolder,           //Command   0x01; requires FilesysSelect obj
    Delete,                 //Command   0x02; requires FilesysSelect obj
    Rename,                 //Command   0x03; requires ExtFileData containing a FilesysSelect obj
}

//Messages relating to the device's flash [mostly legacy]
impl MessageSpecies for FlashMessage{}
pub enum FlashMessage{
    GetCRC,                 //Request   0x00; [LEGACY]
    GetSegment,             //Request   0x01; [LEGACY]
    GetFlashConfig,         //Request   0x02; [LEGACY]
    GetSectorConfig,        //Request   0x03; [LEGACY]
    PutCRC,                 //Response  0x00; [LEGACY]
    PutSegment,             //Response  0x01; [LEGACY]
    PutFlashConfig,         //Response  0x02; [LEGACY]
    PutSectorConfig,        //Response  0x03; [LEGACY]
    WriteSegment,           //Command   0x00; Requires FlashChunk obj
    EraseSector,            //Command   0x01; [LEGACY]
    EnableFlash,            //Command   0x02; [LEGACY]
    DisableFlash,           //Command   0x03; [LEGACY]
    FlashReady,             //Status    0x00; requires FlashMsgArg with IOParamA containing
                            //                  ProgramWaitTime and IOParamB containing ChunkSize
    GetUpgrade,             //Request   0x04; requires FlashMsgArg with IOParamA containing ImageID
                            //                  of the item being upgraded
    PutUpgrade,             //Response  0x04; requires FlashMsgArg with IOParamA containing ModeID
                            //                  of the item device
    GetFlashPrep,           //Request   0x05; requires FlashMsgArg with IOParamA containin ModuleID
                            //                  and IOParamB containing ImageSize
    PutFlashPrep,           //Response  0x05; requires FlashMsgArg with IOParamA containing
                            //                  PrepWaitTime
    FlashComplete,          //Status    0x01; no obj
}

//Messages related to the height measurement module
impl MessageSpecies for HeightMessage{}
pub enum HeightMessage{
    GetHeight,              //Request   0x00;
    GetConfig,              //Request   0x01;
    PutHeight,              //Response  0x00;
    PutConfig,              //Response  0x01;
    WriteConfig,            //Command   0x00;
    ReportHeight,           //Status    0x00;
    StreamHeight,           //Stream    0x00;
}

//Messages related to the HIM module
impl MessageSpecies for HIMMessage{}
pub enum HIMMessage{
    GetConfig,              //Request   0x00; no obj
    PutConfig,              //Response  0x00; requires HIMConfig obj
    WriteMessage,           //Command   0x00; requires HIMData obj
    ReportResponse,         //Status    0x00; requires HIMResponse obj
    ClearMessage,           //Command   0x01; requires HimSelect obj
}

//Messages related to the Heartrate module
impl MessageSpecies for HRMessage{}
pub enum HRMessage{
    GetHR,                  //Request   0x00; requires HRSelect obj
    PutHR,                  //Response  0x00; requires HRData obj with proper select
    GetConfig,              //Request   0x01; requires HRSelect obj
    PutConfig,              //Response  0x01; requires HRConfig obj
    WriteConfig,            //Command   0x00; requires HRConfig obj
    ReportHR,               //Status    0x00; requires HRData obj
    StartAcquire,           //Command   0x01; no obj(?)
    StopAcquire,            //Command   0x02; no obj(?)
    StreamHR,               //Stream    0x00; requires HRData obj
}

//Messages related to interpretation 
impl MessageSpecies for InterpretMessage{}
pub enum InterpretMessage{
    GetInterpretation,      //Request   0x00;
    PutInterpretation,      //Response  0x00;
    WriteInterpretation,    //Command   0x00;
}

//Messages containing lists
impl MessageSpecies for ListMessage{}
pub enum ListMessage{
    GetConfig,              //Request   0x00; Requires ListDesc obj
    PutConfig,              //Response  0x00; requires ListDesc obj
    ReportList,             //Status    0x00; Requires ExtListData obj
    ReportLTR,              //Status    0x01; no obj
    ExecuteLTT,             //Command   0x00; no obj
}

//Messages related to a device's location
impl MessageSpecies for LocationMessage{}
pub enum LocationMessage{
    GetLocation,            //Request   0x00; no obj
    GetConfig,              //Request   0x01; 
    PutLocation,            //Response  0x00; requires LocationConfig obj
    PutConfig,              //Response  0x01;
    WriteConfig,            //Command   0x00; requires LocationConfig obj
    ReportLocation,         //Status    0x00; requires LocationConfig obj
}

impl MessageSpecies for ModuleMessage{}
pub enum ModuleMessage{
    GetModuleDesc,          //Request   0x00; requires ModuleSelect obj
    GetConfig,              //Request   0x01;
    PutModuleDesc,          //Response  0x00; requires a Record containing 1+ ExtModuleData obj
    PutConfig,              //Response  0x01;
    WriteConfig,            //Command   0x00; [LEGACY]
    WriteModuleDesc,        //Command   0x01; requires ModuleData obj
    HWModConnect,           //Event     0x00; [LEGACY]
    HWModDisconnect,        //Event     0x01; [LEGACY]
}

impl MessageSpecies for NetworkMessage{}
pub enum NetworkMessage{
    GetConfig,              //Request   0x00; requires NetworkSelect obj
    PutConfig,              //Response  0x00; requires ExtNetworkData containing 1+ NetworkAdapter obj
    WriteConfig,            //Command   0x00; requires ExtNetworkData containing 1+ NetworkAdapter obj
    WriteATEConfig,         //Command   0x01; requires NetworkAdapter obj [REPAIR ONLY]
    GetData,                //Request   0x01; no obj
    PutData,                //Response  0x01; requires NetworkWiressStatus obj
    ClearBTPairedDevices,   //Command   0x02; no obj
}

impl MessageSpecies for NumericMessage{}
pub enum NumericMessage{
    StartAcquire,           //Command   0x00; Requires NumSelect or ExtNumData containing 2+
                            //                  NumSelect obj
    StopAcquire,            //Command   0x01 Requires NumSelect or ExtNumData containing 2+
                            //                  NumSelect obj
    StreamNumeric,          //Stream    0x00
    WriteConfig,            //Command   0x02; requires NumConfig or NumFloatConfig
    GetConfig,              //Request   0x00; requires NumSelect 
    PutConfig,              //Response  0x00; requires NumConfig, NumFloatConfig, or ExtNumData
                            //                  containing multiple (NumConfig or NumFloatConfig) objects
    ReportConfig,           //Status    0x00; requires NumConfig or NumFloatConfig obj
}

impl MessageSpecies for OrderMessage{}
pub enum OrderMessage{
    Report                  //Status    0x00
}

impl MessageSpecies for PainMessage{}
pub enum PainMessage{
    GetPain,                //Request   0x00;
    GetConfig,              //Request   0x01;
    PutPain,                //Response  0x00;
    PutConfig,              //Response  0x01;
    WriteConfig,            //Command   0x00;
    ReportPain              //Status    0x00;
}

impl MessageSpecies for PatientMessage{}
pub enum PatientMessage{
    GetPatient,             //Request   0x00; may contain PatientSelect obj, default to PatientData obj
    GetConfig,              //Request   0x01; no obj
    PutPatient,             //Response  0x00; may contain PatientEthnic or PatientData obj
    PutConfig,              //Response  0x01; requires patient config object
    WriteConfig,            //Command   0x00; requires patient config object
    WriteData,              //Command   0x01; requires PatientEthnic or PatientData obj
    ReportPatient,          //Status    0x00; Requires PatientEthnic or PatientData obj
    SetPatientState,        //Command   0x02; requires PatientSelect with PatientState enum in Value
    GetPatientState,        //Request   0x02; no obj
    PutPatientState,        //Response  0x02; requires PatientSelect with PatientState enum in Value
    ReportPatientState,     //Status    0x02; requires PatientSelect with PatientState enum in Value
    SetPatientMode,         //Command   0x03; requires PatientSelect with PatientMode enum in Value
    GetPatientMode,         //Request   0x03; no obj
    PutPatientMode,         //Response  0x03; requires PatientSelect with PatientMode enum in Value
    ReportPatientMode,      //Status    0x03; requires PatientSelect with PatientMode enum in Value
    SetPatientExtInfo,      //Command   0x04; requires PatientExtData containing ((PatientEthnic or
                            //                  PatientData obj) AND LocationConfig obj)
    GetPatientExtInfo,      //Request   0x04; may contain PatientSelect obj default to PatientData
                            //                  obj in the PatientEthnic obj
    PutPatientExtInfo,      //Response  0x04; requires PatientExtData containing ((PatientEthnic or
                            //                  PatientData obj) AND LocationConfig obj)
    ReportPatientExtInfo,   //Status    0x04; requires PatientExtData containing ((PatientEthnic or
                            //                  PatientData obj) AND LocationConfig obj)
}

impl MessageSpecies for PatientMotionMessage{}
pub enum PatientMotionMessage{
    //Sensor interface messages
    StartAcquire,           //Command   0x00; no obj
    StopAcquire,            //Command   0x01; no obj
    StreamPatientMotion,    //Stream    0x00; requires PMData obj
    GetSettings,            //Request   0x00; requires PMMsgArg obj
    PutSettings,            //Response  0x00; requires PMMsgArg obj
    WriteSettings,          //Command   0x02; requires PMMsgArg obj
    ReportPatientTurn,      //Status    0x00; requires PMTurn obj

    //Host interface messages
    GetConfig,              //Request   0x01; no obj
    PutConfig,              //Response  0x01; requires PMConfig obj
    WriteConfig,            //Command   0x03; requires PMConfig obj
    ReportConfig,           //Status    0x01; requires PMConfig obj
}

impl MessageSpecies for PrinterMessage{}
pub enum PrinterMessage{
    GetPrinter,             //Request   0x00; no obj
    GetConfig,              //Request   0x01; no obj
    PutPrinter,             //Response  0x00; 
    PutConfig,              //Response  0x01;
    WriteConfig,            //Command   0x00;
    StartPrint,             //Command   0x01;
}

impl MessageSpecies for QueryMessage{}
pub enum QueryMessage{
    GetData,                //Request   0x00; Requires ExtQueryRequest obj
    PutData,                //Response  0x00; Requires ExtQueryResponse obj
}

impl MessageSpecies for RecordMessage{}
pub enum RecordMessage{
    GetRecordCount,         //Request   0x00; requires RecordSelect obj
    GetConfig,              //Request   0x01; [LEGACY]
    PutRecordCount,         //Response  0x00; requires Record obj or RecordService obj
    PutConfig,              //Response  0x01; [LEGACY]
    WriteConfig,            //Command   0x00; [LEGACY]
    WriteRecordCount,       //Command   0x01; requires Record obj or RecordService obj
    ReportRecordCount,      //Status    0x00; requires Record obj or RecordService obj
    ResetCounts,            //Command   0x02; no obj
}

//Messages related to respiration
impl MessageSpecies for RespMessage{}
pub enum RespMessage{
    GetRespiration,         //Request   0x00;
    GetConfig,              //Request   0x01;
    PutRespiration,         //Response  0x00;
    PutConfig,              //Response  0x01;
    WriteConfig,            //Command   0x00;
    ReportRespiration,      //Status    0x00;
    StartAcquire,           //Command   0x01; no obj
    StopAcquire,            //Command   0x02; no obj
    StreamRespiration,      //Stream    0x00; requires SPOCData obj
}

impl MessageSpecies for SessionMessage{}
pub enum SessionMessage{
    GetSession,             //Request   0x00;
    GetConfig,              //Request   0x01;
    GetSessionCount,        //Request   0x02;
    PutSession,             //Response  0x00;
    PutConfig,              //Response  0x01;
    PutSessionCount,        //Response  0x02;
    EraseStoredSessions,    //Command   0x00;
    ReportSession,          //Status    0x00;
    ReportSessionFragment,  //Status    0x01;
    StreamSession,          //Stream    0x00;
    StartAcquire,           //Command   0x01;
    StopAcquire,            //Command   0x02;
}

//Messages related to cardiac output(?)
impl MessageSpecies for SpCOMessage{}
pub enum SpCOMessage{
    GetSpCO,                //Request   0x00; requires SpCOSelect obj
    GetConfig,              //Request   0x01; no obj
    PutSpCO,                //Response  0x00; Requires SpCOData obj
    PutConfig,              //Response  0x01; Requires SpCOData obj
    ReportSpCO,             //Status    0x00; Requires SpCOData obj
    StartAcquire,           //Command   0x00; no obj
    StopAcquire,            //Command   0x01; no obj
    StreamSpCO,             //Stream    0x00; Requires SpCOData obj
}

//Messages related to hemoglobin sensor
impl MessageSpecies for SpHBMessage{}
pub enum SpHBMessage{
    GetSpHB,                //Request   0x00; requires SpHBSelect obj
    GetConfig,              //Request   0x01; may contain SpHBSelect, default to SpHBConfig obj
    PutSpHB,                //Response  0x00; requires SpHBData obj
    PutConfig,              //Response  0x01; requires SpHBConfig or SpHBConfigData obj
    WriteConfig,            //Command   0x00; requires SpHBConfig or SpHBConfigData obj
    ReportSpHB,             //Status    0x00; requires SpHBData obj
    StartAcquire,           //Command   0x01; no obj
    StopAcquire,            //Command   0x02; no obj
    StreamSpHB,             //Stream    0x00; requires SpHBData obj
}

//Messages related to spirometry sensor
impl MessageSpecies for SpiroMessage{}
pub enum SpiroMessage{
    GetConfig,              //Request   0x00;
    PutConfig,              //Response  0x00;
    WriteConfig,            //Command   0x00;
}

//Messages related to methemoglobin sensor
impl MessageSpecies for SpMETMessage{}
pub enum SpMETMessage{
    GetSpMet,               //Request   0x00; requires SpMETSelect obj
    GetConfig,              //Request   0x01; no obj
    PutSpMet,               //Response  0x00; Requires SpMETData obj
    PutConfig,              //Response  0x01; Requires SpMETConfigData obj
    ReportSpMet,            //Status    0x00; Requires SpMETData obj
    StartAcquire,           //Command   0x00; no obj
    StopAcquire,            //Command   0x01; no obj
    StreamSpMet,            //Stream    0x00; Requires SpMETData obj
}

//Messages related to O2 saturation sensor
impl MessageSpecies for SpO2Message{}
pub enum SpO2Message{
    GetSpO2,                //Request   0x00; no obj
    GetConfig,              //Request   0x01; no obj
    PutSpO2,                //Response  0x00; requires SpO2Data obj
    PutConfig,              //Response  0x01; requires SpO2ConfigData obj
    WriteConfig,            //Command   0x00; requires SpO2ConfigData obj
    ReportSpO2,             //Status    0x00; requires SpO2Data obj
    StartAcquire,           //Command   0x01; no obj
    StopAcquire,            //Command   0x02; no obj
    StreamSpO2              //Stream    0x00; requires SpO2Data obj
}

//Messages related to Oxygen Content sensor
impl MessageSpecies for SpOCMessage{}
pub enum SpOCMessage{
    GetSpOC,                //Request   0x00; requires SpOCSelect obj 
    GetConfig,              //Request   0x01; no obj
    PutSpOC,                //Response  0x00; requires SpOCData obj
    PutConfig,              //Response  0x01; requires SpOCConfigData obj
    ReportSpOC,             //Status    0x00; requires SpOCData obj
    StartAcquire,           //Command   0x00; no obj
    StopAcquire,            //Command   0x01; no obj
    StreamSpOC,             //Stream    0x00; Requires SpOCData obj
}

impl MessageSpecies for TOCMessage{}
pub enum TOCMessage{
    GetTOC,                 //Request   0x00;
    PutTOC,                 //Response  0x00;
}

impl MessageSpecies for UIMessage{}
pub enum UIMessage{
    Keyboard,               //Event     0x00; no obj
    GetConfig,              //Request   0x00; requires Numeric obj
    PutConfig,              //Response  0x00; requires Numeric obj
    WriteConfig,            //Command   0x00; requires Numeric obj
    StartCalib,             //Command   0x01; no obj
    StopCalib,              //Command   0x02; no obj
    ReportStatus,           //Status    0x00; Requires UILocation or UICalib obj
    StartUITest,            //Command   0x03; no obj
    StopUITest,             //Command   0x04; no obj
    GetStatus,              //Request   0x01;
    PutStatus,              //Response  0x01; 
}

impl MessageSpecies for UserMessage{}
pub enum UserMessage{
    GetUser,                //Request   0x00;
    PutUser,                //Response  0x00;
}

impl MessageSpecies for WACPMessage{}
pub enum WACPMessage{
    //Stack options: RS232 = 0x01, USB = 0x02, Ethernet 0x03
    StartClient,            //Command   0x00; requires Numeric indicating which stack
    StartHost,              //Command   0x01; requires Numeric indicating which stack
    Connect,                //Command   0x02; requires Numeric indicating which stack
    Disconnect,             //Command   0x03; requires Numeric indicating which stack
    Stop                    //Command   0x04; requires Numeric indicating which stack
}

impl MessageSpecies for WaveMessage{}
pub enum WaveMessage{
    GetWave,                //Request   0x00;
    GetConfig,              //Request   0x01;
    PutWave,                //Response  0x00;
    PutConfig,              //Response  0x01;
    ReportWave,             //Status    0x00;
    WriteWave,              //Command   0x00;
}

impl MessageSpecies for WaveSampleMessage{}
pub enum WaveSampleMessage{
    GetWave,                //Request   0x00; requires WaveSampleSelect obj
    GetConfig,              //Request   0x01; requires WaveSampleSelect obj
    PutWave,                //Response  0x00; requires ExtWaveSampleData containing
                            //                  WaveSampleConfig obj and a wave sample data obj 
    PutConfig,              //Response  0x01; requires ExtWaveSampleData containing 1+
                            //                  WaveSampleConfig obj
    ReportWave,             //Status    0x00; requires ExtWaveSampleData containing
                            //                  WaveSampleConfig obj and a wave sample data obj
    StartAcquire,           //Command   0x00; requires WaveSampleSelect obj
    StopAcquire,            //Command   0x01; requires WaveSampleSelect obj
    WriteConfig,            //Command   0x02; requires ExtWaveSampleData containing 1+
                            //                  WaveSampleConfig obj
    StreamWave,             //Stream    0x00; requires WaveSampleData obj or WaveSampleAnnotate obj
    StreamedLeads,          //Command   0x03; requires WaveSampleSelect containing 1+ wave species
}

impl MessageSpecies for WeightMessage{}
pub enum WeightMessage{
    GetWeight,              //Request   0x00;
    GetConfig,              //Request   0x01;
    PutWeight,              //Response  0x00;
    PutConfig,              //Response  0x01;
    WriteConfig,            //Command   0x00;
    ReportWeight,           //Status    0x00;
    StreamWeight,           //Stream    0x00;
}
