use super::MessageSpecies;

pub enum DeviceMessage{
    GetDeviceDesc,              //Request   0x00; Optionally contains DevMsgArgs with SpeciesSelect set
                                //                  to which subsystem to retrieve
    GetConfig,                  //Request   0x01; Requires DevMsgArgs with SpeciesSelect set to what
                                //                  config to retrieve
    PutDeviceDesc,              //Response  0x00; May contain 1+ DeviceExtData objs (containing 1+
                                //                  DeviceData objs), or a single DeviceData obj.
                                //                  SpeciesSelect should be set to reported subsystem.
    PutConfig,                  //Response  0x01; Requires DeviceConfig obj or DeviceAudioConfig obj
    WriteConfig,                //Command   0x00; Requires DeviceConfig obj or DeviceAudioConfig obj
    WriteDeviceDesc,            //Command   0x01; see PutDeviceDesc for requirements
    EnableDisplay,              //Command   0x02; no obj
    StartDisplayTest,           //Command   0x03; requires DeviceLCDConfig containing state
    Reset,                      //Command   0x04; no obj
    Power,                      //Command   0x05; requires DevMsgArgs with IOParamA set to PowerControl
                                //                  value. Value should be published by devices.
    Beep,                       //Command   0x06; requires DeviceAudioConfig obj
    SaveConfig,                 //Command   0x07; no obj
    RestoreConfig,              //Command   0x08; no obj [factory reset]
    EnableAutoPowerDown,        //Command   0x09; no obj(?)
    SetDisplayContrast,         //Command   0x0a; Requires DeviceLCDConfig
    SetBacklight,               //Command   0x0b; Requires DeviceLCDConfig
    GetPOST,                    //Request   0x02; no obj
    PutPOST,                    //Response  0x02; Requires DevPOSTData obj
    StartDigitalTest,           //Command   0x0c; Requires DevMsgArgs with IOParamA indicating bits
                                //                  correlating to digital lines to signal. Bit-to-line
                                //                  representation is published by each system.
    GetDigitalState,            //Request   0x03; no obj
    PutDigitalState,            //Response  0x03; see StartDigitalTest for requirements
    StopDisplayTest,            //Command   0x0d; no obj
    SetDeviceMode,              //Command   0x0e; Requires DevMsgArgs with IOParamA set to device mode
    GetDeviceMode,              //Request   0x04; no obj
    PutDeviceMode,              //Response  0x04; Requires DevMsgArgs with IOParamA set to device mode
    StartPrinterTest,           //Command   0x0f; no obj
    GetDeviceInventory,         //Request   0x05; no obj
    PutDeviceInventory,         //Response  0x05; requires DevInventory obj
    SetCalib,                   //Command   0x10; requires DevCalConfig obj
    GetCalib,                   //Request   0x06; no obj
    PutCalib,                   //Response  0x06; requires DevCalConfig obj
    SetDatetime,                //Command   0x11; requires DevTimeConfig obj
    GetDatetime,                //Request   0x07; no obj
    PutDatetime,                //Response  0x07; requires DevTimeConfig obj
    NextPatient,                //Command   0x12; no obj(?)
    SetDeviceComMode,           //Command   0x13; requires DevMsgArgs with IOParamA set to ComMode
    GetDeviceComMode,           //Request   0x08; no obj
    PutDeviceComMode,           //Response  0x08; requires DevMsgArgs with IOParamA set to ComMode
    StartHWTest,                //Command   0x14; optional DevMsgArgs with IOParamA set to value for
                                //                  some subsystem
    StopHWTest,                 //Command   0x14; optional DevMsgArgs with IOParamA set to value for
                                //                  some subsystem
    GetHWTestState,             //Request   0x09; optional DevMsgArgs with IOParamA set to value for
                                //                  some subsystem
    PutHWTestState,             //Response  0x09; requires DevPOSTData obj
    ReportHWTestState,          //Status    0x00; requires DevPOSTData obj
    ReportDock,                 //Status    0x01; no obj
    ReportUndock,               //Status    0x02; no obj
    SetAlarmState,              //Command   0x15; no obj(?)
    GetAlarmState,              //Request   0x0a; no obj
    PutAlarmState,              //Response  0x0a; requires DevAlarmState obj containing DevAlarmInfo
    ReportAlarmState,           //Status    0x03; Requires either [DevExtData obj containing
                                //                  DevAlarmInfo] (for host) or [DevAlarmState obj
                                //                  containing DevAlarmInfo] (for device)
    SetMonitorState,            //Command   0x16; requires DevMsgArgs with IOParamA set to MonitorState
    GetMonitorState,            //Request   0x0b; no obj
    PutMonitorState,            //Response  0x0b; requires DevMsgArgs with IOParamA set to MonitorState
    ReportMonitorState,         //Status    0x04; requires DevMsgArgs with IOParamA set to MonitorState
    EndTelemetry,               //Command   0x17; requires DevTelem obj
    StartAcquireAlarmState,     //Command   0x18; no obj(?)
    StopAcquireAlarmState,      //Command   0x19; no obj(?)
    StreamAlarmState,           //Stream    0x00; Requires either [DevExtData obj containing
                                //                  DevAlarmInfo] (for host) or [DevAlarmState obj
                                //                  containing DevAlarmInfo] (for device)
    SetAlarmConfig,             //Command   0x1a; requires DevAlarmConfig obj
    GetAlarmConfig,             //Request   0x0c; no obj(?)
    PutAlarmConfig,             //Response  0x0c; requires DevAlarmConfig obj
    ReportAlarmConfig,          //Status    0x05; requires DevAlarmConfig obj
    SetAlarmMask,               //Command   0x1b; requires DeviceAlarmMask obj
    ReportAlarmResponse,        //Status    0x06; requires DevAlarmInfo obj
    ReportDeviceMode,           //Status    0x07; requires DevMsgArgs with IOParamA set to device mode
    ReportInteractionComplete,  //Status    0x08; requires DevStatus obj
    WriteEndTelemetryDesc,      //Command   0x1c; Requires DevTelem obj
    SetRegister,                //Command   0x1d; requires DevMsgArgs obj
    GetRegister,                //Request   0x0d; requires DevMsgArgs obj
    PutRegister,                //Response  0x0d; requires DevMsgArgs obj
    StartUpdate,                //Command   0x1e; requires DevUpdate obj
    GetServiceMonitorSettings,  //Request   0x0e; optional DevMsgArgs obj. Defaults to
                                //                  DevServiceMonitor obj in response
    SetServiceMonitorSettings,  //Response  0x0e; requires DevServiceMonitor or
                                //                  DevServiceMonitorInfo obj
    PutServiceMonitorSettings,  //Command   0x1f; requires DevServiceMonitor obj
    GetSettingsXferSupported,   //Request   0x0f; no obj [NACK=unsupported]
    PutSettingsXferSupported,   //Response  0x0f; no obj
    StartSettingsXfer,          //Command   0x20; Requires DevSettingsXferOptions obj
    GetSettingsXferStatus,      //Request   0x10; no obj
    PutSettingsXferStatus,      //Response  0x10; Requires DevSettingsXferStatus obj
    ReportSettingsXferStatus,   //Status    0x09; requires DevSettingsXferStatus obj
    ResetDatetime,              //Command   0x21; no obj
    ReportUpdateStatus,         //Status    0x0a; requires DevUpdateStatus
    SetNightMode,               //Command   0x22; requires DevMsgArgs with IOParamA set to bool
    ReportNightModeStatus,      //Status    0x0b; requires DevMsgArgs with IOParamA set to bool
    GetNightModeState,          //Request   0x11; no obj
    PutNightModeState,          //Response  0x11; requires DevMsgArgs with IOParamA set to bool
    StartCallHome,              //Command   0x23; requires DevServiceMonitorInfo obj
    StopCallHome,               //Command   0x24; no obj
    StartTokenScan,             //Command   0x25; no obj
    GetSWUpdateStatus,          //Request   0x12; no obj
    PutSWUpdateStatus,          //Response  0x12; requires DevSWUpdateStatus obj
    GetLightSensor,             //Request   0x13; no obj
    PutLightSensor,             //Response  0x13; requires DevMsgArgs obj
    SetFirstBootSettings,       //Command   0x26; requires DevMsgArgs obj
    GetFirstBootSettings,       //Request   0x14; no obj
    PutFirstBootSettings,       //Response  0x14; requires DevMsgArgs obj
    ReportDeviceInventory,      //Status    0x0c; requires DevInventory obj
    StartBootloader,            //Command   0x27; requires DevMsgArgs obj
    StopBootloader,             //Command   0x28; requires DevMsgArgs obj
    SetLicenseFeatures,         //Command   0x29; requires DevMsgArgs obj
    GetLicenseFeatures,         //Request   0x15; no obj
    PutLicenseFeatures,         //Response  0x14; requires DevMsgArgs obj
}

impl MessageSpecies for DeviceMessage{}
