use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;

#[derive(Clone,Debug)]
pub enum CmdlineSpecies{
    Dispatch,
    Standard,
    Ansi,
    Unicode,
    Description,
}
impl ObjectSpecies for CmdlineSpecies{}

#[derive(Getters,Builder,Clone)]
pub struct CLIExtendedDispatch{
    command_handle:u16,
    view_handle:u16,
    flags:u8,
    error_code:ErrorCode,
    container:&'static dyn Object,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Default,Debug,Clone)]
pub enum ErrorCode{
    #[default]
    None,
    Unsupported,
    ClassSyntax,
    CommandSyntax,
    ParameterSyntax,
}

impl Object for CLIExtendedDispatch{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::CmdLine }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::DataExt }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(CmdlineSpecies::Dispatch) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct CLIData{
    data_format:DataFormat,
    endianness:Endianness,
    data:Vec<u8>,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Default,Debug,Clone)]
pub enum DataFormat{
    #[default]
    None,
    ANSI,
    Unicode,
    Binary,
    MIOB,
}

#[derive(Default,Debug,Clone)]
pub enum Endianness{
    #[default]
    None,
    BigEndian,
    LittleEndian,
}

impl Object for CLIData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::CmdLine }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(CmdlineSpecies::Standard) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct CLIAnsiData{
    arg_count:u32,
    arguments:Vec<char>,    //From u8
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for CLIAnsiData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::CmdLine }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(CmdlineSpecies::Ansi) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct CLIUnicodeData{
    arg_count:u32,
    arguments:Vec<char>,    //From u16
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for CLIUnicodeData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::CmdLine }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(CmdlineSpecies::Unicode) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct CLIDescription{
    data_format:DataFormat,
    delimiter:char, //From u8
    start_sequence:Vec<char>, //from [u8]
    stop_sequence:Vec<char>, //from [u8]
    max_char_limit:u32,
    char_set_page:u16,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for CLIDescription{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::CmdLine }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(CmdlineSpecies::Description) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
