use crate::datatypes;
use chrono;
use super::{Object, ObjectSpecies, RawObject, numeric_objects::StandardNumeric};
use derive_getters::Getters;
use derive_builder::Builder;

#[derive(Clone,Debug)]
pub enum CO2Species{
    Data,       //1,  data
    MsgArgs,    //1,  param
    CalConfig,  //11, config
    Config,     //0,  config
}
impl ObjectSpecies for CO2Species{}

#[derive(Getters,Builder,Clone)]
pub struct Data{
    status:Status,
    init_fail:bool,
    sfm_required:bool,
    sfm_in_progress:bool,
    purging_in_progress:bool,
    filter_line_no_connect:bool,
    check_calib:bool,
    check_flow:bool,
    occlusion_in_gas_line:bool,
    temp_out_of_range:bool,
    pump_off:bool,
    source:Source,
    mode:Mode,
    method:DataMethod,
    et_co2:StandardNumeric,
    fi_co2:StandardNumeric,
    rr:StandardNumeric,
    ipi:StandardNumeric,
    spontaneous_et_co2:StandardNumeric,
    spontaneous_fi_co2:StandardNumeric,
    continuous_co2:StandardNumeric,
    spontaneous_density_5:StandardNumeric,
    spontaneous_density_10:StandardNumeric,
    spontaneous_density_30:StandardNumeric,
    apnea_total:StandardNumeric,
    apnea_10_19:StandardNumeric,
    apnea_20_29:StandardNumeric,
    apnea_30:StandardNumeric,
    desaturation_count:StandardNumeric,
    time:chrono::NaiveDateTime,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum Status{
    #[default]
    DataNotAvailable,   //0
    DataComplete,       //1
    DataFailure,        //2
    DataException,      //3
    DataProcessing,     //4
}

#[derive(Debug,Default,Clone)]
pub enum Source{
    #[default]
    None,               //0
    Unknown,            //1
    BreathDetected,     //2
    Interval,           //3
}

#[derive(Debug,Default,Clone)]
pub enum DataMethod{
    #[default]
    None,               //0
    Unknown,            //1
    Manual,             //2
    Device,             //3
    Remote,             //4
}

#[derive(Debug,Default,Clone)]
pub enum Mode{
    #[default]
    None,               //0
    Unknown,            //1
}

impl Object for Data{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::CO2 }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(CO2Species::Data) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct CO2MsgArg{
    io_param_a:ConfigItem,
    io_param_b:&'static dyn Parameter,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum ConfigItem{
    #[default]
    None,           //0
    BTPS,           //1
    PickMode,       //2
    Standby,        //7
    PatientMode,    //8
    CO2Range,       //9
    ResetParam,     //10
    MeasurementMode,//11
}

pub trait Parameter{}
impl Parameter for BTPS {}
impl Parameter for PickMode {}
impl Parameter for PumpControl {}
impl Parameter for PatientMode {}
impl Parameter for CO2Range {}
impl Parameter for MeasurementMode {}

#[derive(Debug,Default,Clone)]
pub enum BTPS{
    #[default]
    None,   //0
    NoBTPS, //1
    BTPS,   //2
}

#[derive(Debug,Default,Clone)]
pub enum PickMode{
    #[default]
    None,           //0
    BreathToBreath, //1
    TenSeconds,     //2
    TwentySeconds,  //3
    ThirtySeconds,  //4
}

#[derive(Debug,Default,Clone)]
pub enum PumpControl{
    #[default]
    None,       //0
    Disabled,   //1
    Enabled,    //2
}

#[derive(Debug,Default,Clone)]
pub enum PatientMode{
    #[default]
    None,       //0
    Adult,      //1
    Neonate,    //2
    Pediatric1, //3, Pediatric 1-3 yrs
    Pediatric2, //4, Pediatric 3-6 yrs
    Pediatric3, //5, Pediatric 6-12 yrs
}

#[derive(Debug,Default,Clone)]
pub enum CO2Range{
    #[default]
    None,               //0
    NinteyNineMax,      //1
    OneHundredFiftyMax, //2
}

#[derive(Debug,Default,Clone)]
pub enum MeasurementMode{
    #[default]
    None,               //0
    EndTidalCO2,        //1
    ContinuousCO2,      //2
}

impl Object for CO2MsgArg{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::CO2 }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Parameter }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(CO2Species::MsgArgs) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct CO2CalConfig{
    status:CalConfigStatus,
    method:CalConfigMethod,
    gas_concentration:u8,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum CalConfigStatus{
    #[default]
    InProgress,             //0
    NotReady,               //1
    Calculating,            //2
    CompleteError,          //3
    CompleteSuccess,        //4
    UserAbort,              //5
    MeasurementError,       //6
    AZFail,                 //7
    GasConcentrationCheck,  //8
    GasFlowUnstable,        //9
    CalFailure,             //10
}

#[derive(Debug,Default,Clone)]
pub enum CalConfigMethod{
    #[default]
    None,               //0
    RegularCal,         //1
    SpecialFactoryCal,  //2
    CalCheck,           //3
}

impl Object for CO2CalConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::CO2 }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(CO2Species::CalConfig) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct CO2Config{
    display_units:DisplayUnits,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum DisplayUnits{
    #[default]
    MMHG,   //0
    KPA,    //1
}

impl Object for CO2Config{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::CO2 }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(CO2Species::Config) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
