use crate::datatypes;
use chrono;
use super::{error_objects, Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;

#[derive(Clone,Debug)]
pub enum DeviceSpecies{
    Device,                 //0,1,2,255; Data
    POST,                   //3, data
    Inventory,              //4, data
    AlarmInfo,              //5, data
    AlarmState,             //6, data
    Telemetry,              //7, data
    Status,                 //8, data
    SettingsXferSupport,    //9, data
    SettingsXferOptions,    //10, data
    SettingsXferStatus,     //11, data
    SettingsDesc,           //12, data
    UpdateStatus,           //13, data
    SWUpdateStatus,         //14, data
    ProgressDownloading,    //15, data
    ProgressHWModules,      //16, data
    MsgArgs,                //0,1; parameter
    UpdateSelect,           //2, param
    Extended,               //0, extended
    Config,                 //0, config
    AudioConfig,            //1, config
    LCDConfig,              //2, config
    CalConfig,              //3, config
    TimeConfig,             //4, config
    AlarmConfig,            //5, config
    AlarmMaskConfig,        //6, config
    ServiceMonitor,         //7, config
    ServiceMonitorInfo,     //8, config
}
impl ObjectSpecies for DeviceSpecies{}

#[derive(Getters,Builder,Clone)]
pub struct Device{
    time:chrono::NaiveDateTime,
    runtime:u32,
    model_name:String,
    serial_number:String,
    guid:[u8;16],
    model_number:String,
    #[getter(skip)]
    species:&'static DeviceDataSpecies,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone,Copy)]
pub enum DeviceDataSpecies{
    #[default]
    Standard,   //0
    Host,       //1
    Sensor,     //2
    All         //255
}
impl ObjectSpecies for DeviceDataSpecies{}

impl Object for Device{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(*self.species) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct POST{
    guid:[u8;16],
    source_id:String,
    post_status:POSTStatus,
    error_list:Vec<error_objects::Error>, 
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum POSTStatus{
    #[default]
    None,       //0
    InProgress, //1
    Complete,   //2
    Failure,    //3
}

impl Object for POST{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::POST) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct DeviceInventory{
    guid:[u8;16],
    model_name:String,
    model_number:String,
    serial_number:String,
    software_version:String,
    ip_address:[u8;4],
    mac_address:[u8;6],
    ap_mac_address:[u8;6],
    ssid:String,
    rssi:i32,
    connection_mode:ConnectionMode,
    channel:u16,
    wired_mac_address:[u8;6],
    wireless_mac_address:[u8;6],
    wireless_features:WirelessFeatures,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum ConnectionMode{
    #[default]
    None,       //0
    Ethernet,   //1
    Wireless,   //2
}

#[derive(Debug,Default,Clone)]
pub enum WirelessFeatures{
    #[default]
    LamarrBasicSet, //0
    NewmarBasicSet, //1
}

impl Object for DeviceInventory{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::Inventory) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct DeviceMsgArgs{
    species_select:DeviceDataSpecies,
    io_param_a:&'static dyn DeviceEnum,
    io_param_b:&'static dyn DeviceEnum,
    #[getter(skip)]
    species:&'static MsgArgsSpecies,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Debug,Copy)]
pub enum MsgArgsSpecies{
    MsgArgs,    //0
    Result,     //1
}
impl ObjectSpecies for MsgArgsSpecies{}

//Implement this for PowerControl and bit-to-digital line representations
pub trait DeviceEnum{}
impl DeviceEnum for DeviceMode{}
impl DeviceEnum for CommunicationMode{}
impl DeviceEnum for AlarmState{}
impl DeviceEnum for MonitorState{}
impl DeviceEnum for PowerState{}
impl DeviceEnum for SensorID{}

#[derive(Debug,Default,Clone)]
pub enum DeviceMode{
    #[default]
    None,       //0
    Normal,     //1
    Standby,    //2
    Cal,        //3
    Service,    //4
    Demo,       //5
    ATE,        //6
    Error,      //7
    Sleep,      //8
}

#[derive(Debug,Default,Clone)]
pub enum CommunicationMode{
    #[default]
    None,       //0
    Unknown,    //1
    Normal,     //2
    Emulation,  //3
}

#[derive(Debug,Default,Clone)]
pub enum AlarmState{
    #[default]
    None,           //0
    Unknown,        //1
    AudiblePause,   //2
    VisualPause,    //3
}

#[derive(Debug,Default,Clone)]
pub enum MonitorState{
    #[default]
    None,           //0
    Unknown,        //1
    Uninterrupted,  //2
    Interrupted,    //3
}

#[derive(Debug,Default,Clone)]
pub enum PowerState{
    #[default]
    None,       //0
    Charge,     //1
    NoCharge,   //2
    Shutdown,   //3
}

#[derive(Debug,Default,Clone)]
pub enum SensorID{
    #[default]
    None,       //0
    Newmar,     //1
}

impl Object for DeviceMsgArgs{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Parameter }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(*self.species) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct ExtendedDevice{
    device_container:Vec<Device>,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for ExtendedDevice{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::DataExt }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::Extended) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct Config{
    time:chrono::NaiveDateTime,
    standby_interval:u16,
    autooff_interval:u16,
    disable_standby_external_power:bool,
    disable_autooff_external_power:bool,
    audio_beeper_enable:bool,
    language:Language,
    display_date_format:DateFormat,
    display_time_format:TimeFormat,
    supply_voltage:Supply,
    device_id:String,
    char_set_page:u16,
    practice_name:String,
    dst_enabled:bool,
    mode:Mode,
    display_off_interval:u16,
    disable_display_off_eternal_power:bool,
    line_frequency:Frequency,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum Language{
    #[default]
    English,            //0
    German,             //1
    French,             //2
    Spanish,            //3
    Italian,            //4
    Portuguese,         //5
    Dutch,              //6
    Swedish,            //7
    Danish,             //8
    Chinese,            //9
    Finnish,            //10
    Norwegian,          //11
    Polish,             //12
    Korean,             //13
    Greek,              //14
    Czech,              //15
    Estonian,           //16
    Russian,            //17
    Latvian,            //18
    Slovenian,          //19
    Indonesian,         //20
    Arabic,             //21
    Croatian,           //22
    Vianamese,          //23
    BrazillainPortugese,//24
    Thai,               //25
}

#[derive(Debug,Default,Clone)]
pub enum DateFormat{
    #[default]
    MMDDYYYY,   //0
    DDMMYYYY,   //1
    YYYYMMDD,   //2
}

#[derive(Debug,Default,Clone)]
pub enum TimeFormat{
    #[default]
    TwelveHour,     //0
    TwentyFourHour, //1
}

#[derive(Debug,Default,Clone)]
pub enum Supply{
    #[default]
    Supply110V,     //0
    SupplyNon110V,  //1
}

#[derive(Debug,Default,Clone)]
pub enum Mode{
    #[default]
    None,           //0
    ACConnected,    //1
    ACConnectedBase,//2
    Battery,        //3
}

#[derive(Debug,Default,Clone)]
pub enum Frequency{
    #[default]
    None,       //0
    FiftyHz,    //1
    SixtyHz,    //2
}

impl Object for Config{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::Config) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct AudioConfig{
    channel:u8,
    frequency:u16,
    volume:u16,
    state:AudioState,
    duration:u16,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum AudioState{
    #[default]
    None,   //0
    On,     //1
    Off,    //2
    Mute,   //3
}

impl Object for AudioConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::AudioConfig) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct LCDConfig{
    backlight:u8,
    contrast:u8,
    lcd_control:LCDControl,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum LCDControl{
    #[default]
    None,               //0
    Restore,            //1
    PixelsOn,           //2
    PixelsOff,          //3
    PixelsTestPattern1, //4
    PixelsTestPattern2, //5
    PixelsTestPattern3, //6
    PixelsTestPattern4, //7
    PixelsTestPattern5, //8
    PixelsTestPattern6, //9
}

impl Object for LCDConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::LCDConfig) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct CalConfig{
    time:chrono::NaiveDateTime,
    cal_signature:String,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for CalConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::CalConfig) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct TimeConfig{
    time:chrono::DateTime<chrono::Utc>,
    timezone_offset:chrono::offset::FixedOffset, //Raw packet stores in minutes
    dst_offset:chrono::offset::FixedOffset, //Raw packet stores in minutes
    dst_in_effect:bool,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for TimeConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::TimeConfig) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct AlarmInfo{
    rank:u16,
    r#type:Type,
    response_type:ResponseType,
    alarm_description:error_objects::Error,
    hold_off_allow:bool,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum Type{
    #[default]
    None,           //0
    Technical,      //1
    Physiological,  //2
    Informational,  //3
}

#[derive(Debug,Default,Clone)]
pub enum ResponseType{
    #[default]
    None,           //0
    Dismissable,    //1
    Acknowledgable, //2
    Persistent,     //3
}

impl Object for AlarmInfo{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::AlarmInfo) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct AlarmStateObj{
    alarming:bool,
    audio_off:bool,
    audio_off_countdown:u16,
    alarm_off:bool,
    alarm_off_countdown:u16,
    alarm_condition_code:u8,
    alarm_container:Vec<AlarmInfo>,
    night_mode:bool,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for AlarmStateObj{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::AlarmState) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct AlarmConfig{
    audio_pause_time:u16,
    alarm_pause_time:u16,
    max_chars:u16,
    alarm_container:Vec<AlarmInfo>, 
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for AlarmConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::AlarmConfig) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct AlarmMaskConfig{
    alarm_condition_code:u8,
    audio_mask_mode:MaskMode,
    alarm_mask_mode:MaskMode,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum MaskMode{
    #[default]
    None,       //0
    Disable,    //1
    Enable,     //2
    Pause,      //3
}

impl Object for AlarmMaskConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::AlarmMaskConfig) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct DeviceTelemetry{
    mode:TelemetryMode,
    description:String,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum TelemetryMode{
    #[default]
    None,               //0
    Disconnect,         //1
    EndMonitoring,      //2
    Discharge,          //3
    PowerDown,          //4
    RetryTimeout,       //5
    ConnectionRejected, //6
}

impl Object for DeviceTelemetry{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::Telemetry) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct DeviceStatus{
    status:Status,
    msg_class_id:u32,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum Status{
    #[default]
    None,       //0
    Complete,   //1
    Failure,    //2
}

impl Object for DeviceStatus{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::Status) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct UpdateSelect{
    update_token:String,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for UpdateSelect{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Parameter }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::UpdateSelect) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct ServiceMonitor{
    call_home_interval:u32,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for ServiceMonitor{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::ServiceMonitor) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct ServiceMonitorInfo{
    server_ip:[u8;4],
    active:bool,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for ServiceMonitorInfo{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::ServiceMonitorInfo) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct SettingsXferSupport{
    support_level:SupportLevel,
    settings_file_list:Vec<SettingsDesc>, 
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Default,Debug)]
pub enum SupportLevel{
    #[default]
    Basic
}

impl Object for SettingsXferSupport{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::SettingsXferSupport) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct SettingsXferOptions{
    r#role:Role,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Default,Debug)]
pub enum Role{
    #[default]
    None,           //0
    DataRecipient,  //1
    DataSource,     //2
}

impl Object for SettingsXferOptions{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::SettingsXferOptions) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct SettingsXferStatus{
    status:XferStatus,
    error_list:Vec<error_objects::Error>,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Default,Debug)]
pub enum XferStatus{
    #[default]
    None,                   //0
    InProgress,             //1
    Failure,                //2
    Complete,               //3
    CompletePleaseReboot,   //4
    CompleteRebooting,      //5
    FileReady,              //6
}

impl Object for SettingsXferStatus{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::SettingsXferStatus) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}


#[derive(Getters,Builder,Clone)]
pub struct SettingsDesc{
    filename:String,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for SettingsDesc{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::SettingsDesc) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct UpdateStatus{
    selector:UpdateSelect,
    status:UpdStatus,
    error_list:Vec<error_objects::Error>,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Default,Debug)]
pub enum UpdStatus{
    #[default]
    None,           //0
    Complete,       //1
    Failure,        //2
    Exception,      //3
    Processing,     //4
    HostReady,      //5
}

impl Object for UpdateStatus{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::UpdateStatus) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct SWUpdateStatus{
    status:SWStatus,
    extended_status:ExtendedSWStatus,
    update_name:String,
    container:&'static dyn Progress,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Default,Debug)]
pub enum SWStatus{
    #[default]
    None,       //0
    Complete,   //1
    Failure,    //2
    Exception,  //3
    Processing, //4
}

#[derive(Clone,Default,Debug)]
pub enum ExtendedSWStatus{
    #[default]
    None,                   //0
    Pending,                //1
    DownloadManifest,       //2
    DownloadPackage,        //3
    Installing,             //4
    InstallingHWModules,    //5
    DownloadComplete,       //6
}

impl Object for SWUpdateStatus{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::SWUpdateStatus) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

pub trait Progress{}
#[derive(Getters,Builder,Clone)]
pub struct ProgressDownloading{
    total_packages:u16,
    current_packages:u16,
    current_package_name:String,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for ProgressDownloading{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::ProgressDownloading) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
impl Progress for ProgressDownloading{}

#[derive(Getters,Builder,Clone)]
pub struct ProgressHWModules{
    total_modules:u16,
    module_list:String,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for ProgressHWModules{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Device }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(DeviceSpecies::ProgressDownloading) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
impl Progress for ProgressHWModules{}
