use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject, wave_objects, wave_sample_objects};
use derive_getters::Getters;
use derive_builder::Builder;

#[derive(Clone,Debug)]
pub enum ECGSpecies{
    Pacer,          //0, data
    GlobalMeasure,  //1, data
    LeadMeasure,    //2, data
    Status,         //3, data
    RhythmReport,   //32, config
    AutoReport,     //33, config
    MiscConfig,     //34, config
    Config,         //35, config
    Settings,       //36, config
    Select,         //0, parameter
}
impl ObjectSpecies for ECGSpecies{}

#[derive(Getters,Builder,Clone)]
pub struct Config{
    patient_mode:PatientMode,
    trace_restore:bool,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Default,Debug)]
pub enum PatientMode{
    #[default]
    Adult,      //0
    Pediatric,  //1
    Neonate,    //2
}

impl Object for Config{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::ECG }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(ECGSpecies::Config) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct Pacer{
    wave_sample_pacer:wave_sample_objects::WaveSampleDetectConfig,
    wave_data:wave_objects::WaveData,
    wave_config:wave_objects::WaveConfig,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for Pacer{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::ECG }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(ECGSpecies::Pacer) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct GlobalMeasure{
    p_duration:u32,
    pq_duration:u32,
    qrs_duration:u32,
    qt_duration:u32,
    qtc_duration:u32,
    qtd_duration:u32,
    p_axis:u32,
    qrs_axis:u32,
    t_axis:u32,
    heart_rate:u32,
    p_onset:u32,
    p_offset:u32,
    qrs_onset:u32,
    qrs_offset:u32,
    t_offset:u32,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for GlobalMeasure{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::ECG }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(ECGSpecies::GlobalMeasure) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct LeadMeasure{
    lead_identifier:LeadIdentifier, //from u16
    p_pos_amp:i32,
    p_neg_amp:i32,
    q_duration:i32,
    q_amp:i32,
    r_duration:i32,
    r_amp:i32,
    s_duration:i32,
    s_amp:i32,
    r_acc_duration:i32,
    r_acc_amp:i32,
    s_acc_duration:i32,
    s_acc_amp:i32,
    i_duration:i32,
    st_0:i32,
    st_20:i32,
    st_40:i32,
    st_60:i32,
    t_pos_amp:i32,
    t_neg_amp:i32,
    r2_acc_duration:i32,
    r2_acc_amp:i32,
    s2_acc_duration:i32,
    s2_acc_amp:i32,
    k_duration:i32,
    end_t:i32,
    qrs_type:String,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Default,Debug)]
pub enum LeadIdentifier{
    I,      //0x00
    II,     //0x01
    III,    //0x02
    AVR,    //0x03
    AVL,    //0x04
    AVF,    //0x05
    V1,     //0x06
    V2,     //0x07
    V3,     //0x08
    V4,     //0x09
    V5,     //0x0a
    V6,     //0x0b
    V7,     //0x0c
    V3R,    //0x0d
    V5R,    //0x0e
    X,      //0x0f
    Y,      //0x10
    Z,      //0x11
    V4R,    //0x12
    V6R,    //0x13
    V8,     //0x14
    V9,     //0x15
    #[default]
    None,   //0x16
}

impl Object for LeadMeasure{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::ECG }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(ECGSpecies::LeadMeasure) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct RhythmReport{
    lead_arrangement:LeadArrangement,
    default_speed_setting:DefaultSpeedSetting,
    std_lead_order:[RhythmLead;12],
    augmented_lead_order:[RhythmLead;12],
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Default,Debug)]
pub enum LeadArrangement{
    #[default]
    ThreeLead,
    SixLead,
    TwelveLead,
}

#[derive(Clone,Default,Debug)]
pub enum DefaultSpeedSetting{
    #[default]
    TenMMS,
    TwentyFiveMMS,
    FiftyMMS,
}

#[derive(Clone,Default,Debug)]
pub enum RhythmLead{
    I,
    II,
    III,
    AVR,
    AVL,
    AVF,
    V1,
    V2,
    V3,
    V4,
    V5,
    V6,
    #[default]
    Off,
    V7,
    V3R,
    V5R,
    X,
    Y,
    Z,
    V4R,
    V6R,
    V8,
    V9,
}

impl Object for RhythmReport{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::ECG }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(ECGSpecies::RhythmReport) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct AutoReport{
    extended_measurements:bool,
    interp:bool,
    copies_with_int:bool,
    unconf_report:bool,
    abnormal_ecg:bool,
    first_name:bool,
    surname_last_name:bool,
    middle_initial:bool,
    weight:bool,
    height:bool,
    gender:bool,
    race:bool,
    med:bool,
    clinical_history:bool,
    bp:bool,
    comments:bool,
    custom_def_1:bool,
    custom_def_2:bool,
    reason_statements_enabled:bool,
    age_format:AgeFormat,
    lead_arrangement:ExtendedLeadArrangement,
    rhythm_lead_1:RhythmLead,
    rhythm_lead_2:RhythmLead,
    rhythm_lead_3:RhythmLead,
    average_cycle:AverageCycle,
    copies:u16,
    custom_def_1_label:String,
    custom_def_2_label:String,
    physician:bool,
    technician:bool,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Default,Debug)]
pub enum AgeFormat{
    DOB,
    Age,
    #[default]
    Off,
}

#[derive(Clone,Default,Debug)]
pub enum ExtendedLeadArrangement{
    ThreeByFour,
    ThreeByFourOneRhythm,
    ThreeByFourThreeRhythm,
    SixByTwo,
    TwelveByOne,
    SixByTwo50mm,
    SixByTwoExtended,
    TwoBySixOneRhythm,
    #[default]
    NoPrint,
    SixByTwoOneRhythm,
    TwoBySix,
    ThreeByFour25mm5sec,
    ThreeByFour50mm,
    ThreeByFour50mm5sec,
    SixByTwo25mm,
    SixByTwo50mm5sec,
}

#[derive(Clone,Default,Debug)]
pub enum AverageCycle{
    ThreeByFourThreeRhythm,
    SixByTwoSixRhythm,
    #[default]
    NoPrint,
    ThreeByFourTwoRhythm25mm,
    ThreeByFourTwoRhythm,
    ThreeByFourThreeRhythm25mm,
    TwoBySixOneRhythm25mm,
    TwoBySixOneRhythm,
    SixByTwoTwoRhythm,
    SixByTwoOneRhythm25mm,
    SixByTwoOneRhythm,
}

impl Object for AutoReport{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::ECG }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(ECGSpecies::AutoReport) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct MiscConfig{
    lead_config:LeadConfig,
    mains:Frequency,
    auto_export:AutoExport,
    electrode_label:ElectrodeLabel,
    lead_timing:LeadTiming,
    default_gain_setting:DefaultGainSetting,
    default_muscle_filter:bool,
    default_baseline_filter:bool,
    baseline_center:bool,
    autosave:bool,
    auto_report_2:bool,
    print_artifact_report:bool,
    electrode_placement:ElectrodePlacement,
    qtc_method:QTcMethod,
    save_reminder:bool,
    disable_mains:bool,
    ecg_auto_print:bool,
    ecg_set_patient_demographic:bool,
    remote_folder_path:String,
    qrs_tone:QRSTone,
    print_label:PrintLabel,
    pacer_lead:PacerLead,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Default,Debug)]
pub enum LeadConfig{
    #[default]
    Standard,
    Cabrera,
    NEHB,
    Frank,
}

#[derive(Clone,Default,Debug)]
pub enum Frequency{
    None,
    FiftyHz,
    SixtyHz,
    #[default]
    Auto,
}

#[derive(Clone,Default,Debug)]
pub enum AutoExport{
    #[default]
    Off,
    MediaCard,
    FAA,
    Fax,
    Workstation,
    Remote,
}

#[derive(Clone,Default,Debug)]
pub enum ElectrodeLabel{
    #[default]
    AHA,
    IEC,
}

#[derive(Clone,Default,Debug)]
pub enum LeadTiming{
    #[default]
    Simultaneous,
    Sequential,
}

#[derive(Clone,Default,Debug)]
pub enum DefaultGainSetting{
    FivemmPermv,
    TenmmPermv,
    TwentymmPermv,
    #[default]
    Auto,
    TwoPointFivemmPermv,
}

#[derive(Clone,Default,Debug)]
pub enum ElectrodePlacement{
    #[default]
    Standard,
    Alternate,
}

#[derive(Clone,Default,Debug)]
pub enum QTcMethod{
    #[default]
    Bazett,
    Hodges,
    Fridericia,
}

#[derive(Clone,Default,Debug)]
pub enum QRSTone{
    #[default]
    None,
    Freq1KHz,
    Freq2Khz,
    Freq2K67Hz,
    Freq3K2Hz,
    Freq4KHz,
    Freq5K3Hz,
    Freq6K4Hz,
    Freq8KHz,
}

#[derive(Clone,Default,Debug)]
pub enum PrintLabel{
    #[default]
    None,
    PR,
    PQ,
}

#[derive(Clone,Default,Debug)]
pub enum PacerLead{
    None,
    I,
    #[default]
    II,
    III,
    VI,
    V1,
    V2,
    V3,
    V4,
    V5,
    V6,
    V7,
    V8,
    V9,
    V3R,
    V4R,
    V5R,
    V6R,
}

impl Object for MiscConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::ECG }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(ECGSpecies::MiscConfig) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct ECGSelect{
    species_select:u32,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for ECGSelect{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::ECG }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(ECGSpecies::Select) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct ECGSettings{
    active_leads:Vec<ActiveLead>,
    mains_filter:MainsFilter,
    pacer_enabled:bool,
    lead_mode:LeadMode,
    vtach_threshold:u8,
    impedance_respiration:bool,
    asystole_threshold:u16,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Default,Debug)]
pub enum ActiveLead{
    #[default]
    I,
    II,
    III,
    AVR,
    AVL,
    AVF,
    V,
    V1,
    V2,
    V3,
    V4,
    V5,
    V6,
}

#[derive(Clone,Default,Debug)]
pub enum MainsFilter{
    #[default]
    None,
    Off,
    FiftyHz,
    SixtyHz,
}

#[derive(Clone,Default,Debug)]
pub enum LeadMode{
    #[default]
    None,
    Unknown,
    ThreeLead,
    SixLead,
}

impl Object for ECGSettings{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::ECG }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(ECGSpecies::Settings) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct ECGStatus{
    electrode_off_status:ElectrodeStatus,
    arrhythmia_status:Vec<u8>,
    active_leads:Vec<ActiveLead>,
    //container:Vec<Box<dyn Object>>, //Requires additional documentation
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Default,Debug)]
pub enum ElectrodeStatus{
    RL,
    LL,
    RA,
    LA,
    #[default]
    V,
    V1,
    V2,
    V3,
    V4,
    V5,
    V6,
}

impl Object for ECGStatus{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::ECG }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(ECGSpecies::Status) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
