use crate::datatypes;
use chrono;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;

#[derive(Clone,Debug)]
pub enum FilesysSpecies{
    File,
    Description,
    Select,
}
impl ObjectSpecies for FilesysSpecies{}

#[derive(Getters,Builder,Clone)]
pub struct File{
    datetime:chrono::NaiveDateTime,
    storage_medium:StorageMedium,
    file_name:String,
    file_contents:Vec<u8>,
    #[getter(skip)]
    raw_object:RawObject,
}


#[derive(Debug,Default,Clone)]
pub enum StorageMedium{
    #[default]
    Standard,
    RAM,
    Flash,
    SecureDigitalCard,
    CompactFlashCard,
}

impl Object for File{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Filesys }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FilesysSpecies::File) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct FileDescription{
    datetime:chrono::NaiveDateTime,
    size:u32,
    storage_medium:StorageMedium,
    file_name:String,
    r#attributes:Attributes,
    free_space:u32,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum Attributes{
    #[default]
    ReadOnly,
    VolumeRoot,
}

impl Object for FileDescription{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Filesys }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FilesysSpecies::Description) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct FolderDescription{
    path_name:String,
    container:Vec<&'static dyn Object>,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for FolderDescription{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Filesys }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::DataExt }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FilesysSpecies::File) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct FileSelect{
    path_name:String,
    start_index:u16,
    end_index:u16,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for FileSelect{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Filesys }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Parameter }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(FilesysSpecies::Select) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
