use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject, numeric_objects};
use derive_getters::Getters;
use derive_builder::Builder;
use chrono;

#[derive(Clone,Debug,Copy)]
pub enum HRSpecies{
    Data,
    Config,
    Select,
}
impl ObjectSpecies for HRSpecies{}

#[derive(Getters,Builder,Clone)]
pub struct HRData{
    date_time:chrono::NaiveDateTime,
    status:Status,
    extended_status:ExtendedStatus,
    source:Source,
    mode:Mode,
    method:Method,
    hr:numeric_objects::NumParam,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Default,Clone)]
pub enum Status{
    #[default]
    DataNotAvailable,
    DataComplete,
    DataFailure,
    DataException,
    DataProcessing,
}

#[derive(Debug,Default,Clone)]
pub enum ExtendedStatus{
    #[default]
    None,
    Unknown,
    HRFound,
    NoProbe,
    IncorrectProbe,
    SearchingForHR,
    BadProbe,
    ProbeExpired,
}

#[derive(Debug,Default,Clone)]
pub enum Source{
    #[default]
    None,
    Unknown,
    NIBP,
    SPO2,
    ECG,
    PatientMotionBedSensor,
    PatientMotionChairSensor,
}

#[derive(Debug,Default,Clone)]
pub enum Mode{
    #[default]
    None,
    Unknown,
}

#[derive(Debug,Default,Clone)]
pub enum Method{
    #[default]
    None,
    Unknown,
    Manual,
    Device,
    Remote,
    AverageHR,
}

impl Object for HRData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::HR }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies>  { Box::new(HRSpecies::Data) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct HRExtendedData{
    hr_alarms:Vec<numeric_objects::ConfigStandardNumeric>,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for HRExtendedData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::HR }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::DataExt }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies>  { Box::new(HRSpecies::Data) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct HRConfig{
    hr_alarms:numeric_objects::ConfigStandardNumeric,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for HRConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::HR }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies>  { Box::new(HRSpecies::Config) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct HRSelect{
    species_select:u32,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for HRSelect{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::HR }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Parameter }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies>  { Box::new(HRSpecies::Select) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
