use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait NetworkConfig:Downcast{}
#[derive(Clone,Debug)]
pub enum NetworkSpecies{
    WirelessStatus,
    Ethernet,
    LocationID,
    Wifi,
    Workstation,
    Adapter,
    EthernetWifi,
    Bluetooth,
    BluetoothDevice,
    Address,
    WEP,
    EAP,
    PSK,
    All,
    Network,
    Select,
    ExtendedData,
}
impl ObjectSpecies for NetworkSpecies{}

impl NetworkConfig for NetworkAdapterConfig{}
#[derive(Getters,Builder,Clone)]
pub struct NetworkAdapterConfig{
    physical_address:[u8;8],
    description:String,
    media_state:MediaState,
    ethernet_config:NetworkEthernetConfig,
    wifi_configs:Vec<NetworkWifiConfig>,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum MediaState{
    #[default]
    None,
    Disabled,
    Disconnected,
    Connected,
}

impl Object for NetworkAdapterConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Network }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NetworkSpecies::Adapter) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NetworkConfig for NetworkEthernetConfig{}
#[derive(Getters,Builder,Clone)]
pub struct NetworkEthernetConfig{
    fixed_ip_address:[u8;4],
    default_gateway:[u8;4],
    subnet_mask:[u8;4],
    dns_array_1:[u8;4],
    dns_array_2:[u8;4],
    dhcp_enabled:bool,
    auto_dns_enabled:bool,
    wan_timeout_interval:u16,
    network_mode:NetworkMode,
    server_name:String,
    network_id:String,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum NetworkMode{
    #[default]
    Wired,
    Wireless,
}

impl Object for NetworkEthernetConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Network }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NetworkSpecies::Ethernet) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NetworkConfig for NetworkLocationConfig{}
#[derive(Getters,Builder,Clone)]
pub struct NetworkLocationConfig{
    location_id:String,
    unit_id:String,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for NetworkLocationConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Network }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NetworkSpecies::LocationID) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NetworkConfig for NetworkWifiConfig{}
#[derive(Getters,Builder,Clone)]
pub struct NetworkWifiConfig{
    enabled:bool,
    external:bool,
    installed_module:NetworkModule,
    security:Security,
    channel:WifiChannel,
    mode:WifiMode,
    external_port:ExternalPort,
    authentication_mode:AuthenticationMode,
    authentication_type:AuthenticationType,
    encryption_key_size:EncryptKeySize,
    auto_generate_key:bool,
    ssid:String,
    ieee_802_1x_enabled:bool,
    enter_key:String,
    key_index:String,
    channel_5ghz:Wifi5GChannel,
    radio_mode:RadioMode,
    dfs_enabled:bool,
    auth_container:Box<&'static dyn NetworkAuth>,
    #[getter(skip)]
    raw_object:RawObject,
}

pub trait NetworkAuth:Downcast{}

#[derive(Clone,Copy,Debug,Default)]
pub enum NetworkModule{
    #[default]
    None,
    Unknown,
    DPAC,
    Lamarr,
    Newmarr,
    Bootloader,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Security{
    #[default]
    None,
    WPA,
    WEP,
    AES,
    TKIP,
    Disabled,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum WifiChannel{
    #[default]
    None,
    Channel1,
    Channel2,
    Channel3,
    Channel4,
    Channel5,
    Channel6,
    Channel7,
    Channel8,
    Channel9,
    Channel10,
    Channel11,
    Channel12,
    Channel13,
    Channel14,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Wifi5GChannel{
    #[default]
    None,
    Channel36,
    Channel40,
    Channel44,
    Channel48,
    Channel52,
    Channel56,
    Channel60,
    Channel64,
    Channel100,
    Channel104,
    Channel108,
    Channel112,
    Channel116,
    Channel120,
    Channel124,
    Channel128,
    Channel132,
    Channel136,
    Channel140,
    Channel149,
    Channel153,
    Channel157,
    Channel161,
    Channel165,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum WifiMode{
    #[default]
    None,
    WifiDHCP,
    WifiFixedIP,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ExternalPort{
    #[default]
    None,
    Unknown,
    Port1,
    Port2,
    Port3,
    Port4,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum AuthenticationMode{
    #[default]
    None,
    Open,
    Shared,
    WPA,
    WPAPSK,
    WEP,
    WPAEnterprise,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum AuthenticationType{
    #[default]
    None,
    EAP,
    EAPTLS,
    EAPMD5CHAP,
    EAPMSCHAPV2,
    PEAP,
    PEAPTLS,
    PEAPMSCHAPV2,
    LEAP,
    SmartCardOrOther,
    EAPTTLS,
    EAPFast,
    WEP64,
    WEP128,
    PEAPGTC,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum EncryptKeySize{
    #[default]
    None,
    Key64Bit,
    Key128Bit,
    Key256Bit,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum RadioMode{
    #[default]
    None,
    ABGN,
    ABG,
    BGN,
    AN,
    BG,
    AB,
    B,
    A,
}

impl Object for NetworkWifiConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Network }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NetworkSpecies::Wifi) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct NetworkWifiStatus{
    status:Status,
    access_point_mac_address:[u8;6],
    rssi:i32,
    channel:u16,
    link_quality:u8,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Status{
    #[default]
    DataNotAvailable,
    DataComplete,
    DataFailure,
    DataException,
    DataProcessing,
}

impl Object for NetworkWifiStatus{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Network }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NetworkSpecies::WirelessStatus) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NetworkConfig for NetworkWorkstationConfig{}
#[derive(Getters,Builder,Clone)]
pub struct NetworkWorkstationConfig{
    connection_type:ConnectionType,
    remote_port:u16,
    remote_ip_address:[u8;4],
    remote_network_name:String,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ConnectionType{
    #[default]
    None,
    Wired,
    Wireless,
    USBActiveSync,
}

impl Object for NetworkWorkstationConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Network }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NetworkSpecies::Workstation) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NetworkConfig for NetworkBluetoothConfig{}
#[derive(Getters,Builder,Clone)]
pub struct NetworkBluetoothConfig{
    state:BTState,
    profile:Profile,
    version:String,
    timeout:u16,
    power:u8,
    connection_type:BTConnectionType,
    lease_time:u32,
    connections:Vec<NetworkAdapterConfig>,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum Profile{
    #[default]
    None,
    Unknown,
    A2DDP,
    AVRCP,
    BIP,
    BPP,
    FTP,
    GAVDP,
    GAP,
    GOEP,
    HCRP,
    HP,
    HID,
    OPP,
    PAN,
    SPP,
    SIM,
    VDP,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum BTState{
    #[default]
    None,
    Unknown,
    EnabledPaired,
    EnabledNotPaired,
    DisabledPaired,
    DisabledNotPaired,
    Pairing,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum BTConnectionType{
    #[default]
    None,
    Unknown,
    Temporary,
    Permanent,
    Factory,
}

impl Object for NetworkBluetoothConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Network }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NetworkSpecies::Bluetooth) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NetworkConfig for NetworkBluetoothDeviceConfig{}
#[derive(Getters,Builder,Clone)]
pub struct NetworkBluetoothDeviceConfig{
    physical_address:[u8;6],
    friendly_name:String,
    connectable:bool,
    discoverable:bool,
    ios_seed_name:String,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for NetworkBluetoothDeviceConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Network }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NetworkSpecies::BluetoothDevice) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NetworkConfig for NetworkAddressConfig{}
#[derive(Getters,Builder,Clone)]
pub struct NetworkAddressConfig{
    ip_address:[u8;4],
    port:u16,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for NetworkAddressConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Network }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NetworkSpecies::Address) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NetworkConfig for NetworkWEPConfig{}
impl NetworkAuth for NetworkWEPConfig{}
#[derive(Getters,Builder,Clone)]
pub struct NetworkWEPConfig{
    wep_key_size:u16,
    wep_key_1:String,
    wep_key_2:String,
    wep_key_3:String,
    wep_key_4:String,
    wep_index_key:u16,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for NetworkWEPConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Network }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NetworkSpecies::WEP) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
impl NetworkConfig for NetworkEAPConfig{}
impl NetworkAuth for NetworkEAPConfig{}
#[derive(Getters,Builder,Clone)]
pub struct NetworkEAPConfig{
    username:String,
    password:String,
    auth_server:EAPAuthServer,
    inner_auth:EAPInnerAuth,
    validate_server_enabled:bool,
    pac_provision_type:PACProvision,
    outer_auth:EAPOuterAuth,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum EAPAuthServer{
    #[default]
    None,
    AuthServerACS,
    AuthServerSBR,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum EAPInnerAuth{
    #[default]
    None,
    Auto,
    PAP,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum PACProvision{
    #[default]
    None,
    Manual,
    Auto,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum EAPRoamingType{
    #[default]
    None,
    PMKCaching,
    OKC,
    Radius80211,
    CCKM,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum EAPOuterAuth{
    #[default]
    None,
    EAPTLS,
    EAPMSCHAPV2,
    PEAPTLS,
    PEAPMSCHAPV2,
    EAPTTLS,
    EAPFast,
    PEAPGTC,
}

impl Object for NetworkEAPConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Network }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NetworkSpecies::EAP) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NetworkAuth for NetworkPSKConfig{}
impl NetworkConfig for NetworkPSKConfig{}
#[derive(Getters,Builder,Clone)]
pub struct NetworkPSKConfig{
    psk_key:String,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for NetworkPSKConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Network }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NetworkSpecies::PSK) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct NetworkSelect{
    species_select:u32,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for NetworkSelect{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Network }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Parameter }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NetworkSpecies::Select) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct NetworkExtendedData{
    container:Box<&'static dyn NetworkConfig>,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for NetworkExtendedData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Network }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::DataExt }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NetworkSpecies::ExtendedData) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
