use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject, numeric_objects};
use derive_getters::Getters;
use derive_builder::Builder;
use downcast_rs::Downcast;

pub trait NIBP:Downcast{}

#[derive(Clone,Debug)]
pub enum NIBPSpecies{
    StandardData,
    Data,
    EngineeringChunk,
    EventChunk,
    HardwareState,
    Lock,
    RawData,
    PressureData,
    Status,
    DataModified,
    StandardConfig,
    ConfigData,
    CalConfig,
    ParamData,
    Select,
    ChunkSelect,
    ExtendedData,
}
impl ObjectSpecies for NIBPSpecies{}

impl NIBP for NIBPStatus{}
#[derive(Getters,Builder,Clone)]
pub struct NIBPStatus{
    module_state:ModuleState,
    error:u16,
    hardware_states:Vec<HardwareState>,
    safety_states:Vec<SafetyState>,
    cuff_pressure:i16,
    battery_voltage:i16,
    ambient_temp:i16,
    status_type:StatusType,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum ModuleState{
    #[default]
    Idle,
    Autozero,
    Inflating,
    Stabilizing,
    Bleeding,
    Dumping,
    Repump,
    Error,
    Boot,
    POST,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum HardwareState{
    #[default]
    ValveClosed,
    PumpOn,
    ECGPulseRecieved,
    SavingCal,
    SavingSettings,
    SingleLumenHoseConnected,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum SafetyState{
    #[default]
    BelowResidual,
    LongAutoTimeClear,
    StatAutoTimerClear,
    FailsafeFault,
    AutoZeroFault,
    TransducerFault,
}

#[derive(Clone,Copy,Debug,Default)]
pub enum StatusType{
    #[default]
    None,
    Primary,
    Secondary,
}

impl Object for NIBPStatus{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::NIBP }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NIBPSpecies::Status) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NIBP for NIBPCalConfig{}
#[derive(Getters,Builder,Clone)]
pub struct NIBPCalConfig{
    cal_type:StatusType,
    cal_signature:i32,
    cal_date_time:chrono::NaiveDateTime,
    module_config:i16,
    span_correction_factor:i16,
    zero_correction_factor:i16,
    battery_cal:i16,
    temp_cal:i16,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Copy,Clone,Default)]
pub enum CalType{
    #[default]
    None,
    Primary,
    Secondary,
}

impl Object for NIBPCalConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::NIBP }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NIBPSpecies::CalConfig) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NIBP for NIBPChunk{}
#[derive(Getters,Builder,Clone)]
pub struct NIBPChunk{
    tag:String,
    chunk_id:u16,
    chunk_index:u16,
    chunk_data:[u8;100],
    #[getter(skip)]
    is_event_chunk:bool,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for NIBPChunk{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::NIBP }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { 
        match self.is_event_chunk{
            true => Box::new(NIBPSpecies::EventChunk),
            false => Box::new(NIBPSpecies::EngineeringChunk) 
        }
    }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NIBP for NIBPHWState{}
#[derive(Getters,Builder,Clone)]
pub struct NIBPHWState{
    hardware:Hardware,
    state:i16,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Copy,Clone,Default)]
pub enum Hardware{
    #[default]
    PrimaryValve,
    SecondaryValve,
    Pump,
}

impl Object for NIBPHWState{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::NIBP }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NIBPSpecies::HardwareState) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NIBP for NIBPLock{}
#[derive(Getters,Builder,Clone)]
pub struct NIBPLock{
    lock:Lock,
    state:i8,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Copy,Clone,Default)]
pub enum Lock{
    #[default]
    AmbientTempCheck,
    PrimarySafetyCheck,
    SafetyCheck,
    SingleLumenCheck,
    ADChannel,
    FailsafeTestMode,
}

impl Object for NIBPLock{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::NIBP }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NIBPSpecies::Lock) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NIBP for NIBPRawData{}
#[derive(Getters,Builder,Clone)]
pub struct NIBPRawData{
    raw_5v:u32,
    raw_1v:u32,
    raw_2v:u32,
    raw_4v:u32,
    raw_battery:i16,
    raw_ambient_temp_count:u32,
    raw_primary_transducer_count:u32,
    raw_secondary_transducer_count:u32,
    raw_secondary_battery_count:u32,
    raw_secondary_correction_factor:u32,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for NIBPRawData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::NIBP }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NIBPSpecies::RawData) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NIBP for NIBPDataParam{}
#[derive(Getters,Builder,Clone)]
pub struct NIBPDataParam{
    target_presssure:i16, //50-280
    patient_mode:PatientMode,
    cycle_mode:CycleMode,
    algorithm:Algorithm,
    disable_neo_check:i8,
    first_reading:bool,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Copy,Clone,Default)]
pub enum PatientMode{
    #[default]
    Default,
    Adult,
    Pediatric,
    Neonate,
}

#[derive(Debug,Copy,Clone,Default)]
pub enum CycleMode{
    #[default]
    Manual,
    LongAuto,
    ShortAuto,
}

#[derive(Debug,Copy,Clone,Default)]
pub enum Algorithm{
    #[default]
    Default,
    FastBP,
    StepBP,
    NeoBP,
}

impl Object for NIBPDataParam{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::NIBP }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Parameter }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NIBPSpecies::Data) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NIBP for NIBPConfigData{}
#[derive(Getters,Builder,Clone)]
pub struct NIBPConfigData{
    default_adult_target_pressure:i16,
    default_pediatric_target_pressure:i16,
    default_neonate_target_pressure:i16,
    pediatric_max_pressure_limit:i16,
    residual_max_pressure_limit:i16,
    default_patient_mode:DefaultPatientMode,
    default_adult_alg:DefaultAlgorithm,
    default_pediatric_alg:DefaultAlgorithm,
    default_neonate_alg:DefaultAlgorithm,
    sao_enabled:bool,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Copy,Clone,Default)]
pub enum DefaultPatientMode{
    #[default]
    Adult,
    Pediatric,
    Neonate,
}

#[derive(Debug,Copy,Clone,Default)]
pub enum DefaultAlgorithm{
    #[default]
    FastBP,
    StepBP,
    NeoBP,
}

impl Object for NIBPConfigData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::NIBP }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NIBPSpecies::ConfigData) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NIBP for NIBPData{}
#[derive(Getters,Builder,Clone)]
pub struct NIBPData{
    status:Status,
    systolic:numeric_objects::StandardNumeric,
    diastolic:numeric_objects::StandardNumeric,
    mean_artery_pressure:numeric_objects::StandardNumeric,
    pulse_rate:numeric_objects::StandardNumeric,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Copy,Clone,Default)]
pub enum Status{
    #[default]
    Ok,
    Abort,
    NotAvailable,
    Splat,
    Artifact,
    LowInflation,
    KinkedTube,
    EarlyDump,
    NotEnoughData,
    PatientInvalid,
    ReinflateTooLate,
    Bleed,
    LeakDetected,
    SafetyFault,
    ZeroFault,
}

impl Object for NIBPData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::NIBP }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NIBPSpecies::Data) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NIBP for NIBPDataModified{}
#[derive(Getters,Builder,Clone)]
pub struct NIBPDataModified{
    nibp_data:NIBPData,
    status:ModifiedStatus,
    time:chrono::NaiveDateTime,
    mode:Mode,
    patient_position:PatientPosition,
    cuff_size:CuffSize,
    cuff_location:CuffLocation,
    source:Source,
    method:Method,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Copy,Clone,Default)]
pub enum ModifiedStatus{
    #[default]
    DataNotAvailable,
    DataComplete,
    DataFailure,
    DataException,
    DataProcessing,
}

#[derive(Debug,Copy,Clone,Default)]
pub enum Mode{
    #[default]
    None,
    Unknown,
    Adult,
    Neonatal,
    Pediatric,
}

#[derive(Debug,Copy,Clone,Default)]
pub enum PatientPosition{
    #[default]
    None,
    Unknown,
    Lying,
    Sitting,
    Standing,
}

#[derive(Debug,Copy,Clone,Default)]
pub enum CuffSize{
    #[default]
    None,
    Unknown,
    Neonatal1,
    Neonatal2,
    Neonatal3,
    Neonatal4,
    Neonatal5,
    SmallInfant,
    Infant,
    SmallChild,
    Child,
    SmallAdult,
    Adult,
    AdultLong,
    LargeAdult,
    LargeAdultLong,
    Thigh,
}

#[derive(Debug,Copy,Clone,Default)]
pub enum CuffLocation{
    #[default]
    None,
    Unknown,
    LeftArm,
    RightArm,
    LeftLeg,
    RightLeg,
}

#[derive(Debug,Copy,Clone,Default)]
pub enum Source{
    #[default]
    None,
    Unknown,
    CVSM,
    MODB,
    MODF,
    POEM,
    MODG,
}

#[derive(Debug,Copy,Clone,Default)]
pub enum Method{
    #[default]
    None,
    Unknown,
    Manual,
    Device,
    Remote,
    FastBP,
    StepBP,
    NeonatalStepBP,
    LinearDeflationBP,
    AverageBP
}

impl Object for NIBPDataModified{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::NIBP }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NIBPSpecies::DataModified) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NIBP for NIBPChunkSelect{}
#[derive(Getters,Builder,Clone)]
pub struct NIBPChunkSelect{
    chunk:u32,
    index:u32,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for NIBPChunkSelect{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::NIBP }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Parameter }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NIBPSpecies::ChunkSelect) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NIBP for NIBPSpeciesSelect{}
#[derive(Getters,Builder,Clone)]
pub struct NIBPSpeciesSelect{
    species:u32,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for NIBPSpeciesSelect{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::NIBP }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Parameter }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NIBPSpecies::Select) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NIBP for NIBPStandardData{}
#[derive(Getters,Builder,Clone)]
pub struct NIBPStandardData{
    time:chrono::NaiveDateTime,
    status:Status,
    systolic:i16,
    diastolic:i16,
    map:i16,
    heartrate:u16,
    source:Source,
    mode:Mode,
    method:Method,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for NIBPStandardData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::NIBP }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NIBPSpecies::StandardData) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

impl NIBP for NIBPConfig{}
#[derive(Getters,Builder,Clone)]
pub struct NIBPConfig{
    display_units:DisplayUnits,
    map_display:bool,
    #[getter(skip)]
    raw_object:RawObject,
}

#[derive(Debug,Copy,Clone,Default)]
pub enum DisplayUnits{
    #[default]
    MmHg,
    KPa,
}

impl Object for NIBPConfig{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::NIBP }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NIBPSpecies::StandardConfig) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct NIBPExtendedData{
    container:Vec<&'static dyn NIBP>,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for NIBPExtendedData{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::NIBP }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::DataExt }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NIBPSpecies::ExtendedData) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
