use crate::datatypes;
use super::{Object, ObjectSpecies, RawObject};
use derive_getters::Getters;
use derive_builder::Builder;

#[derive(Clone,Debug)]
pub enum NumericSpecies{
    Standard,           //0
    NIBPSys,            //1
    NIBPDia,            //2
    NIBPMap,            //3
    NIBPPR,             //4
    IBPSys,             //5
    IBPDia,             //6
    IBPMap,             //7
    IBPPR,              //8
    SPO2PR,             //9
    SPO2Sat,            //10
    SPO2PI,             //11
    CO2RR,              //12
    CO2ET,              //13
    CO2FI,              //14
    RR,                 //15
    PR,                 //16
    IPI,                //17
    THB,                //18
    THBAlt,             //19
    Pain,               //20
    PatientMotion,      //21
    AttemptedTurnCount, //22
    TurnTimer,          //23
    Float,              //31
    Temp,               //32
    Long,               //127
    Weight,             //128
    Select,             //0; genus param
}
impl ObjectSpecies for NumericSpecies{}

#[derive(Getters,Builder,Clone)]
pub struct StandardNumeric{
    value:i16,
    valid_range:bool,
    over_range:bool,
    under_range:bool,
    undetermined_range:bool,
    no_alarm:bool,
    upper_limit_alarm:bool,
    lower_limit_alarm:bool,
    other_alarm:bool,
    numeric_active:bool,
    continuous:bool,
    spot_old:bool,
    spot_recent:bool,
    spot_new:bool,
    dubious:bool,
    #[getter(skip)]
    species:NumericSpecies,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for StandardNumeric{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Numeric }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(self.species.clone()) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct LongNumeric{
    value:i32,
    valid_range:bool,
    over_range:bool,
    under_range:bool,
    undetermined_range:bool,
    no_alarm:bool,
    upper_limit_alarm:bool,
    lower_limit_alarm:bool,
    other_alarm:bool,
    numeric_active:bool,
    continuous:bool,
    spot_old:bool,
    spot_recent:bool,
    spot_new:bool,
    dubious:bool,
    #[getter(skip)]
    species:NumericSpecies,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for LongNumeric{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Numeric }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(self.species.clone()) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct FloatNumeric{
    value:f32,
    valid_range:bool,
    over_range:bool,
    under_range:bool,
    undetermined_range:bool,
    no_alarm:bool,
    upper_limit_alarm:bool,
    lower_limit_alarm:bool,
    other_alarm:bool,
    numeric_active:bool,
    continuous:bool,
    spot_old:bool,
    spot_recent:bool,
    spot_new:bool,
    dubious:bool,
    #[getter(skip)]
    species:NumericSpecies,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for FloatNumeric{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Numeric }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Data }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(self.species.clone()) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct ConfigStandardNumeric{
    value:i16,
    over_range_val:i16,
    under_range_val:i16,
    upper_limit_alarm_val:i16,
    lower_limit_alarm_val:i16,
    upper_limit_alarm_state:State,
    lower_limit_alarm_state:State,
    over_range:bool,
    under_range:bool,
    undetermined_range:bool,
    no_alarm:bool,
    upper_limit_alarm:bool,
    lower_limit_alarm:bool,
    other_alarm:bool,
    numeric_active:bool,
    continuous:bool,
    spot_old:bool,
    spot_recent:bool,
    spot_new:bool,
    dubious:bool,
    #[getter(skip)]
    species:NumericSpecies,
    #[getter(skip)]
    raw_object:RawObject
}

#[derive(Default,Debug,Clone)]
pub enum State{
    #[default]
    None,       //0
    Unknown,    //1
    Enabled,    //2
    Disabled,   //3
}

impl Object for ConfigStandardNumeric{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Numeric }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(self.species.clone()) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct ConfigLongNumeric{
    value:i32,
    over_range_val:i32,
    under_range_val:i32,
    upper_limit_alarm_val:i32,
    lower_limit_alarm_val:i32,
    upper_limit_alarm_state:State,
    lower_limit_alarm_state:State,
    valid_range:bool,
    over_range:bool,
    under_range:bool,
    undetermined_range:bool,
    no_alarm:bool,
    upper_limit_alarm:bool,
    lower_limit_alarm:bool,
    other_alarm:bool,
    numeric_active:bool,
    continuous:bool,
    spot_old:bool,
    spot_recent:bool,
    spot_new:bool,
    dubious:bool,
    #[getter(skip)]
    species:NumericSpecies,
    #[getter(skip)]
    raw_object:RawObject
}

impl Object for ConfigLongNumeric{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Numeric }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(self.species.clone()) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct ConfigFloatNumeric{
    value:f32,
    over_range_val:f32,
    under_range_val:f32,
    upper_limit_alarm_val:f32,
    lower_limit_alarm_val:f32,
    upper_limit_alarm_state:State,
    lower_limit_alarm_state:State,
    valid_range:bool,
    over_range:bool,
    under_range:bool,
    undetermined_range:bool,
    no_alarm:bool,
    upper_limit_alarm:bool,
    lower_limit_alarm:bool,
    other_alarm:bool,
    numeric_active:bool,
    continuous:bool,
    spot_old:bool,
    spot_recent:bool,
    spot_new:bool,
    dubious:bool,
    #[getter(skip)]
    species:NumericSpecies,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for ConfigFloatNumeric{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Numeric }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Config }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(self.species.clone()) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}

#[derive(Getters,Builder,Clone)]
pub struct NumParam{
    species_select:u8,
    io_param_a:u32,
    #[getter(skip)]
    raw_object:RawObject,
}

impl Object for NumParam{
    fn get_family(&self) -> datatypes::Family { datatypes::Family::Numeric }
    fn get_genus(&self) ->  datatypes::Genus { datatypes::Genus::Parameter }
    fn get_species(&self) -> Box<dyn super::ObjectSpecies> { Box::new(NumericSpecies::Select) }
    fn get_raw_object(&self) -> RawObject { self.raw_object.clone() }
}
