use std::{time::Duration, 
          io::{BufReader,Read},
          error::Error};
use serialport::SerialPort;
use crate::datatypes;
use crate::packet_parse;


const BAUD_RATE:u32 = 115200;
const SERIAL_TIMEOUT: Duration = Duration::from_millis(250);

pub struct TTY{
    tty: Box<dyn SerialPort>,
    serial: String
}

impl std::fmt::Debug for TTY{
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result{
        let relative_location:String;
        match self.tty.name() {
            Some(absolute_location) => {
                match absolute_location.rsplit_once('/'){
                    Some((_,serial_device_name)) => relative_location = serial_device_name.to_string(),
                    None => relative_location = "unknown".to_string()
                }
            },
            None => relative_location = "unknown".to_string()
        };
        f.debug_struct("TTY")
            .field("Serial port location",&relative_location)
            .finish()
    }
}

impl TTY{
    pub fn new(serial_location:&str,baudrate:Option<u32>,timeout:Option<Duration>) -> Result<Self,Box<dyn Error>> {
        match serialport::new(serial_location,baudrate.unwrap_or(BAUD_RATE))
                            .timeout(timeout.unwrap_or(SERIAL_TIMEOUT))
                            .open(){
            Ok(tty) => {
                Ok(TTY{tty,serial:String::new()})
            },
            Err(error) => {
                Err(Box::new(error))
            }
        }
    }

    pub fn send_packet(&mut self,write_buffer:datatypes::Packet) -> bool{
        let mut output = false;

        //Packet header
        //Preamble starts each packet
        output = self.tty.write(b"\x17\01\0c").is_ok();

        output = self.tty.write(&write_buffer.get_length().to_be_bytes()).is_ok();
        match &write_buffer.get_port(){
            datatypes::Port::RENDEZVOUS => {
                output = self.tty.write(b"\x01\x19").is_ok();
            },
            datatypes::Port::WACP => {
                output = self.tty.write(b"\x01\x1a").is_ok();
            }
        }

        //Message information
        output = self.tty.write(&write_buffer.get_msg().get_raw_id().to_be_bytes()).is_ok();
        output = self.tty.write(&write_buffer.get_msg().get_length().to_be_bytes()).is_ok();
        output = self.tty.write(&write_buffer.get_msg().get_bitmask().to_be_bytes()).is_ok();
        output = self.tty.write(&write_buffer.get_msg().get_object_size().to_be_bytes()).is_ok();

        //Object information
        if write_buffer.get_msg().get_object_size() > 0{
            let object = write_buffer.get_msg().get_object().unwrap();
            output = self.tty.write(&object.get_raw_object().get_raw_id().to_be_bytes()).is_ok();
            output = self.tty.write(&object.get_raw_object().get_size().to_be_bytes()).is_ok();
            output = self.tty.write(&object.get_raw_object().get_version().to_be_bytes()).is_ok();
            output = self.tty.write(&object.get_raw_object().get_bitmask().to_be_bytes()).is_ok();
            //Payload is anonymous right now; as this evolves, it may get more info
            output = self.tty.write(&object.get_raw_object().get_payload().as_slice()).is_ok();
            output = self.tty.write(&object.get_raw_object().get_crc().to_be_bytes()).is_ok();
        }
        output = self.tty.write(&write_buffer.get_msg().get_crc().to_be_bytes()).is_ok();
        output = self.tty.write(&write_buffer.get_crc().to_be_bytes()).is_ok();
        return output
    }

    pub fn recv_packets(&mut self) -> Vec<datatypes::Packet> {
        let mut reader = BufReader::new(&mut self.tty);
        let mut read_buffer:Vec<u8> = Vec::new();
        _ = reader.read_to_end(&mut read_buffer);
        packet_parse::read_packets(read_buffer)
    }

    pub fn set_serial_id(&mut self, serial:String) { self.serial = serial }

    pub fn get_serial_id(&self) -> String { self.serial.clone() }
}
